<?php
/**
 * Plugin Name: senangPay recurring
 * Description: Enable online payments using credit or debit cards and online banking. Currently senangPay service is only available to businesses that reside in Malaysia.
 * Version: 1.0.0
 * Author: senangPay
 * Author URI: https://senangpay.my/
 **/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

# Include senangPay Class and register Payment Gateway with WooCommerce
add_action( 'plugins_loaded', 'senangpay_recurring_init', 0 );

function senangpay_recurring_init() {
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}

	include_once( 'src/senangpay_recurring.php' );

	add_filter( 'woocommerce_payment_gateways', 'add_senangpay_recurring_to_woocommerce' );
	function add_senangpay_recurring_to_woocommerce( $methods ) {
		$methods[] = 'senangpay_recurring';

		return $methods;
	}
}

# Add custom action links
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'senangpay_recurring_links' );

function senangpay_recurring_links( $links ) {
	$plugin_links = array(
		'<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=senangpay_recurring' ) . '">' . __( 'Settings', 'senangpay_recurring' ) . '</a>',
	);

	# Merge our new link with the default ones
	return array_merge( $plugin_links, $links );
}

// add_action( 'init', 'senangpay_recurring_check_response', 15 );

function senangpay_recurring_check_response() {
	# If the parent WC_Payment_Gateway class doesn't exist it means WooCommerce is not installed on the site, so do nothing
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}

	include_once( 'src/senangpay_recurring.php' );

	$senangpay = new senangpay_recurring();
	$senangpay->check_senangpay_recurring_response();
}

function senangpay_recurring_hash_error_msg( $content ) {
	return '<div class="woocommerce-error">The data that we received is invalid. Thank you.</div>' . $content;
}

function senangpay_recurring_payment_declined_msg( $content ) {
	return '<div class="woocommerce-error">The payment was declined. Please check with your bank. Thank you.</div>' . $content;
}

function senangpay_recurring_success_msg( $content ) {
	return '<div class="woocommerce-info">The payment was successful. Thank you.</div>' . $content;
}

add_action('rest_api_init', 'recurring_return_route');

function recurring_return_route() {
    register_rest_route('recurring', 'return', array(
        'methods' => 'GET',
        'callback' => 'senangpay_recurring_check_response',
            )
    );
}

add_action('rest_api_init', 'recurring_callback_route');

function recurring_callback_route() {
    register_rest_route('recurring', 'callback', array(
        'methods' => 'POST',
        'callback' => 'senangpay_recurring_check_response',
            )
    );
}
add_filter( 'woocommerce_order_number', 'wc_set_order_number', 10, 2 );
function wc_set_order_number( $order_id, $order ) {

    // Get the order number (custom meta data)
    $order_number = $order->get_meta('_order_number'); 

    return empty($order_number) ? $order_id : $order_number;
}