<?php
/**
 * senangPay Payment Gateway Class
 */
class senangpay_recurring extends WC_Payment_Gateway {
	function __construct() {
		$this->id = "senangpay_recurring";

		$this->method_title = __( "senangPay Recurring", 'senangPay Recurring' );

		$this->method_description = __( "senangPay Recurring Payment Gateway Plug-in for WooCommerce", 'senangPay Recurring' );

		$this->title = __( "senangPay Recurring", 'senangPay Recurring' );

		$this->hash_type = 'md5';

		$this->environment_mode = 'live';

		if ($this->environment_mode == 'sandbox') 
			$this->icon = 'https://sandbox.senangpay.my/public/img/logo-senangpay-wc.png';
		else
			$this->icon = 'https://app.senangpay.my/public/img/logo-senangpay-wc.png';

		$this->has_fields = true;

		$this->init_form_fields();

		$this->init_settings();

		foreach ( $this->settings as $setting_key => $value ) {
			$this->$setting_key = $value;
		}

		if ( is_admin() ) {
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
				$this,
				'process_admin_options'
			) );
		}
	}

	# Build the administration fields for this specific Gateway
	public function init_form_fields() {
		$this->form_fields = array(
			'enabled'        => array(
				'title'   => __( 'Enable / Disable', 'senangPay Recurring' ),
				'label'   => __( 'Enable this payment gateway', 'senangPay Recurring' ),
				'type'    => 'checkbox',
				'default' => 'no',
			),
			'title'          => array(
				'title'    => __( 'Title', 'senangPay Recurring' ),
				'type'     => 'text',
				'desc_tip' => __( 'Payment title the customer will see during the checkout process.', 'senangPay Recurring' ),
				'default'  => __( 'senangPay Recurring', 'senangPay Recurring' ),
			),
			'description'    => array(
				'title'    => __( 'Description', 'senangPay Recurring' ),
				'type'     => 'textarea',
				'desc_tip' => __( 'Payment description the customer will see during the checkout process.', 'senangPay Recurring' ),
				'default'  => __( 'Pay securely using your credit card or online banking through senangPay Recurring.', 'senangPay Recurring' ),
				'css'      => 'max-width:350px;'
			),
			'universal_form' => array(
				'title'    => __( 'Merchant ID', 'senangPay Recurring' ),
				'type'     => 'text',
				'desc_tip' => __( 'This is the merchant ID that you can obtain from profile page in senangPay', 'senangPay' ),
			),
			'secretkey'      => array(
				'title'    => __( 'Secret Key', 'senangPay Recurring' ),
				'type'     => 'text',
				'desc_tip' => __( 'This is the secret key that you can obtain from profile page in senangPay', 'senangPay' ),
			),
			'hash_type'         => array(
				'title'       => __( 'Hash Type', 'senangPay Recurring' ),
				'type'        => 'select',
				'class'       => 'wc-enhanced-select',
				'description' => __( 'Choose whether you wish to use encryption md5 or sha256.', 'senangPay Recurring' ),
				'default'     => 'md5',
				'desc_tip'    => true,
				'options'     => array(
					'md5'          => __( 'md5', 'senangPay Recurring' ),
					'sha256' => __( 'sha256', 'senangPay Recurring' ),
				),
			),
			'environment_mode'         => array(
				'title'       => __( 'Environment Mode', 'senangPay Recurring' ),
				'type'        => 'select',
				'class'       => 'wc-enhanced-select',
				'description' => __( 'Choose whether you wish to use sandbox or production mode.', 'senangPay Recurring' ),
				'default'     => 'live',
				'desc_tip'    => true,
				'options'     => array(
					'live'	=> __( 'Live', 'senangPay Recurring' ),
					'sandbox'	=> __( 'Sandbox', 'senangPay Recurring' )
				),
			)
		);
	}


	# Submit payment
	public function process_payment( $order_id ) {
		# Get this order's information so that we know who to charge and how much
		$customer_order = wc_get_order( $order_id );
		$items = $customer_order->get_items();
		if (count($items)>1){
			wc_add_notice( __('Error: ', 'woothemes') . 'Sorry, order item for recurring order cannot be more than 1.' , 'error' );
		} else {
			foreach ( $items as $item ) {
				if ($item->get_quantity() > 1){
					wc_add_notice( __('Error: ', 'woothemes') . 'Sorry, quantity for recurring order cannot be more than 1.' , 'error' );
				} else {
					$product_id = $item->get_product_id();
					$product = wc_get_product( $product_id );

					if (empty($product->get_attribute('recurring'))){
						wc_add_notice( __('Error: ', 'woothemes') . 'Sorry, selected product is not recurring.' , 'error' );
					} else {
						# Prepare the data to send to senangPay

						$order_id = $customer_order->get_id();
						$recurring_id = $product->get_attribute('recurring');
						$name     = $customer_order->get_billing_first_name() . ' ' . $customer_order->get_billing_last_name();
						$email    = $customer_order->get_billing_email();
						$phone    = $customer_order->get_billing_phone();

						if ($this->hash_type == 'md5') 
							$hash_value = md5( $this->secretkey . $recurring_id . $order_id );
						else
							$hash_value = hash('sha256', $this->secretkey . $recurring_id . $order_id);	

						$post_args = array(
							'order_id' => $order_id,
							'recurring_id' => $recurring_id,
							'hash'     => $hash_value,
							'name'     => $name,
							'email'    => $email,
							'phone'    => $phone
						);

						# Format it properly using get
						$senangpay_args = '';
						foreach ( $post_args as $key => $value ) {
							if ( $senangpay_args != '' ) {
								$senangpay_args .= '&';
							}
							$senangpay_args .= $key . "=" . $value;
						}

						if ($this->environment_mode == 'sandbox') 
							$environment_mode_url = 'https://api.sandbox.senangpay.my/recurring/payment/';
						else
							$environment_mode_url = 'https://api.senangpay.my/recurring/payment/';

						return array(
							'result'   => 'success',
							'redirect' =>  $environment_mode_url . $this->universal_form . '?' . $senangpay_args
						);
					}
				}
			}
		}
	}

	public function check_senangpay_recurring_response() {
		global $woocommerce;

		$data = json_decode(file_get_contents('php://input'), true);
		
		#post for callback function
		if ( isset($data['recurring_id']) && isset( $data['status_id'] ) && isset( $data['order_id'] ) && isset( $data['msg'] ) && isset( $data['transaction_id'] ) && isset( $data['hash'] ) ) {

			$order = wc_get_order( $data['order_id'] );
			$order_id = $order->get_id();
			if ( $order && $order_id != 0 ) {

				# Check if the data sent is valid based on the hash value
				if ($this->hash_type == 'md5'){
					$hash_value = md5( $this->secretkey . $data['status_id'] . $data['order_id'] . $data['transaction_id'] . $data['msg']);
				} else {
					$hash_value = hash('sha256',  $this->secretkey . $data['status_id'] . $data['order_id'] . $data['transaction_id'] . $data['msg']);
				}

				usort($data['payment_details'], function ($a, $b) {
					return $a['payment_date'] <=> $b['payment_date'];
				});
				
				if ( $hash_value == $data['hash'] ) {
					$int = 0;
					foreach ($data['payment_details'] as $payment_detail){
						$int++;
						if ($payment_detail['payment_transaction_reference'] && $payment_detail['payment_status'] ==  'paid'){
							#check if date is same date with order
							if ($order->get_date_created()->format('d/m/Y')==$payment_detail['payment_date']){
								if ( $payment_detail ==  'paid') {
									if ( strtolower( $order->get_status() ) == 'pending' || strtolower( $order->get_status() ) == 'processing' ) {
										# only update if order is pending
										if ( strtolower( $order->get_status() ) == 'pending' ) {
											$order->payment_complete();
											$order->add_order_note( 'Payment successfully made through senangPay. Transaction reference is ' . $payment_detail['payment_transaction_reference'] );
										}
									}
								}
							} else {
								#when not first payment
								$recurring_display_id = $order_id.'_'.$int;
								
								$args = array(
									'meta_key'   => '_order_number',
									'meta_value' => $recurring_display_id
								);

								$query = new WC_Order_Query( $args );
								$recurring_orders = $query->get_orders();
								# check if order id for recurring already have, update if exist
								if ($recurring_orders){

									$recurring_order = $recurring_orders[0];
									$recurring_order_id = $recurring_order->get_id();
									if ($payment_detail['payment_transaction_reference'] && $payment_detail['payment_status'] ==  'paid'){
										if ( strtolower( $recurring_order->get_status() ) == 'pending' || strtolower( $recurring_order->get_status() ) == 'processing' ) {
											# only update if order is pending
											if ( strtolower( $recurring_order->get_status() ) == 'pending' ) {
												$recurring_order->payment_complete();
												$recurring_order->add_order_note( 'Payment successfully made through senangPay. Transaction reference is ' . $payment_detail['payment_transaction_reference'] );
											}
										}
									}
								} else {
									#if order id recurring not exist, create new order
									$recurring_order = new WC_Order();
									$recurring_order->set_date_created($payment_detail['payment_date']);


									$recurring_order->update_meta_data('_order_number', $recurring_display_id);

									$address_billing = array(
										'first_name' => $order->get_billing_first_name(),
										'last_name'  => $order->get_billing_last_name(),
										'company'    => $order->get_billing_company(),
										'email'      => $order->get_billing_email(),
										'phone'      => $order->get_billing_phone(),
										'address_1'  => $order->get_billing_address_1(),
										'address_2'  => $order->get_billing_address_2(),
										'city'       => $order->get_billing_city(),
										'state'      => $order->get_billing_state(),
										'postcode'   => $order->get_billing_postcode(),
										'country'    => $order->get_billing_country()
									);
									$recurring_order->set_address( $address_billing, 'billing' );
									
									$address_shipping = array(
										'first_name' => $order->get_shipping_first_name(),
										'last_name'  => $order->get_shipping_last_name(),
										'company'    => $order->get_shipping_company(),
										'address_1'  => $order->get_shipping_address_1(),
										'address_2'  => $order->get_shipping_address_2(),
										'city'       => $order->get_shipping_city(),
										'state'      => $order->get_shipping_state(),
										'postcode'   => $order->get_shipping_postcode(),
										'country'    => $order->get_shipping_country()
									);
									$recurring_order->set_address( $address_shipping, 'shipping' );

									foreach ( $order->get_items() as $item_id => $item ) {
										$product_id = $item->get_product_id();
										$recurring_order->add_product( get_product($product_id), 1);
										}
										
									$recurring_order->calculate_totals();
									$recurring_order->set_payment_method($order->get_payment_method());
									
									if ($payment_detail['payment_transaction_reference'] && $payment_detail['payment_status'] ==  'paid'){
										$recurring_order->payment_complete();
										$recurring_order->add_order_note( 'Automatic payment successfully made through senangPay recurring. Original order id is '. $order_id .'. Transaction reference is ' . $payment_detail['payment_transaction_reference'] );
									}
									$recurring_order->save();
								}
							}
						}
					}
				} else {
					add_filter( 'the_content', 'senangpay_recurring_hash_error_msg' );
				}
			}
			echo 'OK';
			exit();
		} else if ( isset( $_REQUEST['status_id'] ) && isset( $_REQUEST['order_id'] ) && isset( $_REQUEST['msg'] ) && isset( $_REQUEST['transaction_id'] ) && isset( $_REQUEST['hash'] ) ) {
			#get from return url
			$order = wc_get_order( $_REQUEST['order_id'] );

			$order_id = $order->get_id();

			if ( $order && $order_id != 0 ) {
				# Check if the data sent is valid based on the hash value				
				if ($this->hash_type == 'md5'){
					$hash_value = md5( $this->secretkey . $_REQUEST['status_id'] . $_REQUEST['order_id'] . $_REQUEST['transaction_id'] . $_REQUEST['msg']);
				} else {
					$hash_value = hash('sha256',  $this->secretkey . $_REQUEST['status_id'] . $_REQUEST['order_id'] . $_REQUEST['transaction_id'] . $_REQUEST['msg']);
				}

				if ( $hash_value == $_REQUEST['hash'] ) {

					if ( $_REQUEST['status_id'] == 1 || $_REQUEST['status_id'] == '1' ) {
						if ( strtolower( $order->get_status() ) == 'pending' || strtolower( $order->get_status() ) == 'processing' ) {
							# only update if order is pending
							if ( strtolower( $order->get_status() ) == 'pending' ) {
								$order->payment_complete();

								$order->add_order_note( 'Payment successfully made through senangPay. Transaction reference is ' . $_REQUEST['transaction_id'] );
							}

							# redirect to order receive page
							wp_redirect( $order->get_checkout_order_received_url() );
							exit();
						}
					} else {
						if ( strtolower( $order->get_status() ) == 'pending' ) {
							$order->add_order_note( 'Payment was unsuccessful' );
							add_filter( 'the_content', 'senangpay_recurring_payment_declined_msg' );
						}
					}
				} else {
					add_filter( 'the_content', 'senangpay_recurring_hash_error_msg' );
				}
			}
		}
	}

	# Validate fields, do nothing for the moment
	public function validate_fields() {
		return true;
	}

	# Check if we are forcing SSL on checkout pages, Custom function not required by the Gateway for now
	public function do_ssl_check() {
		if ( $this->enabled == "yes" ) {
			if ( get_option( 'woocommerce_force_ssl_checkout' ) == "no" ) {
				echo "<div class=\"error\"><p>" . sprintf( __( "<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href=\"%s\">forcing the checkout pages to be secured.</a>" ), $this->method_title, admin_url( 'admin.php?page=wc-settings&tab=checkout' ) ) . "</p></div>";
			}
		}
	}

	/**
	 * Check if this gateway is enabled and available in the user's country.
	 * Note: Not used for the time being
	 * @return bool
	 */
	public function is_valid_for_use() {
		return in_array( get_woocommerce_currency(), array( 'MYR' ) );
	}

}
